<?php

namespace Service;

use DateTime;
use Exception;
use WebSocket\Client;
use Zeedhi\Framework\Socket\Server\Bridge;
use Zeedhi\Framework\Socket\Server\Client\ClientInterface;
use Zeedhi\Framework\Socket\Server\Event\ConnectionEvent;
use Zeedhi\Framework\Socket\Server\Payload;
use Psr\Log\LoggerInterface;
use Zeedhi\Framework\Socket\Server\Connection\ConnectionManagerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Util\ElapsedTime;

use function PasswordCompat\binary\check;

class CheckForUpdate extends Bridge {

	protected $KDSManagerController;
	protected $KDSTransferController;
	protected $entityManager;
	protected $instanceManager;
	protected $connection;
	protected $util;
	protected $query;
	protected $webSocketUpdateHandler;
	protected $messageService;
	protected $orderStash;
	protected $environment;
	private $wsport;
	private $socketClient;
	private $KDSLogs;
	private $logTimer;
	private $pos0000196913;
	private $connected;
	private $OLDDATE;
	protected $lastUpdate = null;
	protected $frozen = false;
	protected $freezeTime = null;
	public $countStatistics = false;
	private $currentDay = null;
	private $currentHour = null;
	private $hora_limpeza_estatistica = null;
	private $intervalo_limpeza = null;


	public function __construct(
		ConnectionManagerInterface $connectionManager,
		EventDispatcherInterface $eventDispatcher,
		LoggerInterface $logger,
		\Controller\KDSManager $KDSManagerController,
		\Controller\KDSTransfer $KDSTransferController,
		\Doctrine\ORM\EntityManager $entityManager,
		\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
		\Doctrine\DBAL\Connection $connection,
		\Util\Util $util,
		\Util\Query $query,
		\Service\WebSocketUpdateHandler $webSocketUpdateHandler,
		\Service\MessageService $messageService,
		\Util\OrderStash $orderStash,
		\Lib\Environment $environment,
		$wsport,
		$KDSLogs,
		$pos0000196913) {
		parent::__construct($connectionManager, $eventDispatcher, $logger);
		$this->KDSManagerController = $KDSManagerController;
		$this->KDSTransferController = $KDSTransferController;
		$this->entityManager = $entityManager;
		$this->instanceManager = $instanceManager;
		$this->util = $util;
		$this->connection = $entityManager->getConnection();
		$this->query = $query;
		$this->webSocketUpdateHandler = $webSocketUpdateHandler;
		$this->messageService = $messageService;
		$this->wsport = $wsport;
		$this->KDSLogs = $KDSLogs;
		$this->orderStash = $orderStash;
		$this->environment = $environment;
        $this->logTimer = array();
        $this->pos0000196913 = $pos0000196913;
	}

	public function connectedWebSocket() {
		if(!isset($this->socketClient) || !$this->socketClient->isConnected()){
			$this->socketClient = new Client("ws://localhost:".$this->wsport."/");
			$packageToWebSocket = array(
				'event' => 'loginBrowser',
				'data'  => array(
					'CDFILIAL' => 'CHECK',
	 				'CDSETOR'  => '1',
	 				'SECURITYKEY' => 'CHECKFORUPDATES'
	 			)
			);
			$this->sendToBrowser($packageToWebSocket);
		}
	}

	public static function getSubscribedEvents() {
		return array(
			'freezeUpdates'        => array('freezeUpdates'),
			'unfreezeUpdates'      => array('unfreezeUpdates')
		);
	}

	public function checkForUpdates() {
		$this->connectedWebSocket();
		$localQuery = $this->query;

		try{
			$this->checkAllEntries();

			$filiais = $this->connection->fetchAll($localQuery::GET_FILIAL);

			if (count($filiais) > 1) {
				// echo "Existe mais de uma filial cadastrada na base. Nao foi possivel inicializar o serviço.";
				$this->util->logException("Existe mais de uma filial cadastrada na base. Nao foi possivel inicializar o serviço.");
			} else {
				$CDFILIAL = $filiais[0]['CDFILIAL'];
				$noErro = true;

				//Processar posteriormente
				$afterProcess = Array();

				do {
					$this->KDSLogs->log('UPDATE_STATUS_PEDIDO_KDS_TEMP_BY_ID',[],dirname(__FILE__).'\\'.basename(__FILE__));
					// $cfu_time = ElapsedTime::create("CFU");
					// $cfu_time->start();
					// Deleta os dados temporários das estatísticas
					$this->clearStatistics();
					// limpa os pedidos que não foram expedidos
					$this->clearOldOrders();

					try {
						$this->frozen = $this->getFrozen();
						if(!$this->frozen){
							$this->connected = false;
							$params = array(
								':CDFILIAL' => $CDFILIAL
							);

							//Trata os pedidos que estão em espera
							$pedido_kds_temp = $this->connection->fetchAll($localQuery::SELECT_PEDIDO_KDS_TEMP, $params);
							$pedido_kds_temp_espera = $this->connection->fetchAll($localQuery::SELECT_PEDIDO_KDS_TEMP_ESPERA, $params);
							for ($i = 0; $i < count($pedido_kds_temp_espera); $i++) {
								$this->connected = true;
                                $this->connection->beginTransaction();
								$item_pedido_kds_temp_espera = $pedido_kds_temp_espera[$i];
								if ($item_pedido_kds_temp_espera['TP_PEDIDO'] === 'COE') {
									$newOrdersEspera = $this->connection->fetchAll($localQuery::GET_SALES_COM_ESPERA, $item_pedido_kds_temp_espera);
								} else {
									$newOrdersEspera = $this->connection->fetchAll($localQuery::GET_SALES_VEN_ESPERA, $item_pedido_kds_temp_espera);
								}
								if (!empty($newOrdersEspera)) {
									$itemsForInsertion = $this->KDSManagerController->pedidoFosGenerator($newOrdersEspera);
									if (!empty($itemsForInsertion)) {
										$this->KDSManagerController->handleProductsForDisplay($itemsForInsertion);
									}
								}
								$this->connection->executeQuery($localQuery::DELETE_PEDIDO_KDS_TEMP_ESPERA_BY_NRPEDIDOFOS, $item_pedido_kds_temp_espera);
								$this->connection->executeQuery($localQuery::DELETE_ITPEDIDFOSREL_ITENS_ESPERA, $item_pedido_kds_temp_espera);
								$this->connection->commit();
								$this->connected = false;
							}
							

							// só executa a rotina de inserção caso existam novos pedidos
							if (!empty($pedido_kds_temp)) {

								for($i=0; $i<count($pedido_kds_temp); $i++){

                                    $this->connected = true;
                                    $this->connection->beginTransaction();

									$item_pedido_kds_temp = $pedido_kds_temp[$i];

                                    $this->KDSLogs->log('UPDATE_STATUS_PEDIDO_KDS_TEMP_BY_ID',[],dirname(__FILE__).'\\'.basename(__FILE__));
									$this->connection->executeQuery($localQuery::UPDATE_STATUS_PEDIDO_KDS_TEMP_BY_ID, $item_pedido_kds_temp);

                                    // VEN = Venda - COM = Comanda
									if($item_pedido_kds_temp['TP_PEDIDO'] == 'VEN'){
                                        $this->KDSLogs->log('GET_SALES_VEN',[],dirname(__FILE__).'\\'.basename(__FILE__));
										$newOrders = $this->connection->fetchAll($localQuery::GET_SALES_VEN, $item_pedido_kds_temp);
									} else if ($item_pedido_kds_temp['TP_PEDIDO'] == 'ROD') {
										$this->KDSLogs->log('GET_SALES_ROD',[],dirname(__FILE__).'\\'.basename(__FILE__));

										// seleciona outros itens do mesmo pedido de rodizio para não serem tratados mais de uma vez
										$itensRodizio = array_filter($pedido_kds_temp, function ($item) use ($item_pedido_kds_temp) {
											return $item['CDFILIAL'] == $item_pedido_kds_temp['CDFILIAL'] && $item['NRVENDAREST'] == $item_pedido_kds_temp['NRVENDAREST'] && $item['NRCOMANDA'] == $item_pedido_kds_temp['NRCOMANDA'] && $item['COD_PEDIDO_KDS_TEMP'] != $item_pedido_kds_temp['COD_PEDIDO_KDS_TEMP'];
										});

										// remover de $pedido_kds_temp os itens de itensRodizio
										foreach ($itensRodizio as $key => $value) {
											$index = array_search($value, $pedido_kds_temp);
											unset($pedido_kds_temp[$index]);
										}

										// seleciona os itens do pedido de rodizio para serem tratados
										$newOrders = $this->connection->fetchAll($localQuery::GET_SALES_ROD, $item_pedido_kds_temp);

										// deleta de PEDIDO_KDS_TEMP os itens de rodizio que já estão na fila para serem tratados
										foreach ($itensRodizio as $itemRodizio) {
											$this->connection->executeQuery($localQuery::DELETE_PEDIDO_KDS_TEMP_BY_ID, $itemRodizio);
										}
									} else {
                                        $this->KDSLogs->log('GET_SALES_COM',[],dirname(__FILE__).'\\'.basename(__FILE__));
										$newOrders = $this->connection->fetchAll($localQuery::GET_SALES_COM, $item_pedido_kds_temp);
									}
                                    
									if (!empty($newOrders)) {
										// insere os itens na PEDIDOFOS e ITPEDIDOFOS
                                        $this->KDSLogs->log('pedidoFosGenerator',[],dirname(__FILE__).'\\'.basename(__FILE__));
										$itemsForInsertion = $this->KDSManagerController->pedidoFosGenerator($newOrders);
										if (!empty($itemsForInsertion)) {
											// ordena, define hora de exibição e insere na ITPEDIDOFOS
                                            $this->KDSLogs->log('handleProductsForDisplay',[],dirname(__FILE__).'\\'.basename(__FILE__));
											$this->KDSManagerController->handleProductsForDisplay($itemsForInsertion);
										}
									}

									//Deleta pedido que já foi processado
                                    $this->KDSLogs->log('DELETE_PEDIDO_KDS_TEMP_BY_ID',[],dirname(__FILE__).'\\'.basename(__FILE__));
									$this->connection->executeQuery($localQuery::DELETE_PEDIDO_KDS_TEMP_BY_ID, $item_pedido_kds_temp);
                                    
                                    $this->KDSLogs->log('commit DELETE_PEDIDO_KDS_TEMP_BY_ID',[],dirname(__FILE__).'\\'.basename(__FILE__));
									$this->connection->commit();

									if(empty($newOrders)){
										array_push($afterProcess, $item_pedido_kds_temp['NRCOMANDA']);
									}

									$this->connected = false;
								}

							}

                            
							$this->connected = true;
							$this->connection->beginTransaction();

							//ISSOS PODEM SER REQUISICOES
							// verifica se tem alguma transferência a ser checkForTransfersfeita
                            $this->KDSLogs->log('checkForTransfers',[],dirname(__FILE__).'\\'.basename(__FILE__));
							$itemsToDeleteInFront = $this->checkForTransfers($CDFILIAL);

							// deleta os itens transferidos do front
                            $this->KDSLogs->log('deleteItemsInFront',[],dirname(__FILE__).'\\'.basename(__FILE__));
							$this->deleteItemsInFront($itemsToDeleteInFront);

							// $init_mcr = microtime(true);
							// faz as alterações solicitadas no web socket
                            $this->KDSLogs->log('handleUpdates',[],dirname(__FILE__).'\\'.basename(__FILE__));
							$this->webSocketUpdateHandler->handleUpdates($this->socketClient);

							//die();

							//$total = number_format(microtime(true)-$init_mcr, 9);
							// $init_mcr = microtime(true);


							// $total = number_format(microtime(true)-$init_mcr, 9);
							// $init_mcr = microtime(true);
                            $this->KDSLogs->log('messageService',[],dirname(__FILE__).'\\'.basename(__FILE__));
							$this->messageService->getMessages();

                            $this->KDSLogs->log('commit messageService',[],dirname(__FILE__).'\\'.basename(__FILE__));
                            $this->connection->commit();

							//ISSO PROVAVELMENTE CONTINUA
							// procura por mudanças no KDS (status, liberações, etc)
                            $this->KDSLogs->log('handleOrdersToSend',[],dirname(__FILE__).'\\'.basename(__FILE__));
                            $this->handleOrdersToSend($CDFILIAL);

                            $this->KDSLogs->log('afterProcess',[],dirname(__FILE__).'\\'.basename(__FILE__));
							// Trigger updates for orders with payment confirmed
							if(sizeof($afterProcess)){
								foreach ($afterProcess as $key => $item) {
									$current_param = Array("NRCOMANDA" => $item);
									$ped_vnd_nrcomanda = $this->connection->fetchAll($localQuery::CHECK_NRPEDIDOFOS_NRSEQVENDA_BY_NRCOMANDA, $current_param);
									if((count($ped_vnd_nrcomanda) > 0)){
										$this->connection->beginTransaction();
										$this->uptateTimeToShow($ped_vnd_nrcomanda); // Atualiza a hora de exibição do pedido para o momento em que ele foi pago
										$this->connection->executeQuery($localQuery::UPDATE_DTULTATU_ITPEDIDOFOS_BY_NRCOMANDA, $current_param);
										$this->connection->commit();
										unset($afterProcess[$key]);
									} else if (empty($ped_vnd_nrcomanda[0]['NRPEDIDOFOS'])){
										unset($afterProcess[$key]);
									}
								}
							}
						} else {
							$this->util->logExecution('KDS em pausa devido a função "Congelar Pedidos"');
						}
			
						// $cfu_time->end();
						//Resolve o problema dos pedidos presos no caixa automático que já foram expedidos
                        $this->KDSLogs->log('GET_NRVENDAREST_COUNT_PEDCONC_TOP',[],dirname(__FILE__).'\\'.basename(__FILE__));
						$countPedConc = $this->connection->fetchAssoc($localQuery::GET_NRVENDAREST_COUNT_PEDCONC_TOP);
						if($countPedConc['CONTEMPEDIDO'] > 0){
                            
                            $this->KDSLogs->log('GET_NRVENDAREST_COUNT_PEDCONC',[],dirname(__FILE__).'\\'.basename(__FILE__));
                            $countPedConc = $this->connection->fetchAll($localQuery::GET_NRVENDAREST_COUNT_PEDCONC);
                            
                            foreach ($countPedConc as $key => $countPedConcValue) {
                                $current_param = Array("CDFILIAL" => $countPedConcValue['CDFILIAL'], "NRCOMANDA" => $countPedConcValue['NRCOMANDA'], "NRVENDAREST" => $countPedConcValue['NRVENDAREST']);
                                $log_current_param = json_encode($current_param);

                                $this->KDSLogs->log("UPDATE_IDIMPCUPEXP_PEDCONC $log_current_param",[],dirname(__FILE__).'\\'.basename(__FILE__));
                                $this->connection->executeQuery($localQuery::UPDATE_IDIMPCUPEXP_PEDCONC, $current_param);
                            }
                            
						}

						sleep(1);
					} catch(\Exception $e){
						$this->util->logException($e->getMessage());
						if ($this->connected){
							$this->connection->rollBack();
						}
						$this->socketClient = null;
						$this->lastUpdate = null;
						$this->orderStash->deleteAllOrders();
						sleep(1);
						$this->connectedWebSocket();
						$noErro = false;
						$this->checkForUpdates();
					}
				} while ($noErro);
			}
		} catch(\Exception $e){
			$this->util->logException($e->getMessage());
			sleep(5);
			$this->checkForUpdates();
		}
	}

	public function microtime_float(){
	    list($usec, $sec) = explode(" ", microtime());
	    return ((float)$usec + (float)$sec);
	}

	public function freezeUpdates(ConnectionEvent $msgEvent){
		$this->frozen = true;
		$localQuery = $this->query;
		$this->freezeTime = $this->connection->fetchAssoc("SELECT GETDATE() AS DATA")['DATA'];
		$params = array(':IDKDSHABILITADO' => 'N');
		$this->connection->executeQuery($localQuery::UPDATE_IDKDSHABILITADO, $params);
	}

	public function unfreezeUpdates(ConnectionEvent $msgEvent){
		$this->frozen = false;
		$this->lastUpdate = $this->freezeTime;
		$localQuery = $this->query;

		$params = array(':HORACONGELAMENTO' => new \Datetime($this->freezeTime), ':NRPEDIDOFOS' => 'NPRODUZIDO');
		$this->connection->executeQuery($localQuery::UPDATE_OLD_REQUESTS_VENDA, $params);
		$this->connection->executeQuery($localQuery::UPDATE_OLD_REQUESTS_COMANDA, $params);

		$params = array(':IDKDSHABILITADO' => 'S');
		$this->connection->executeQuery($localQuery::UPDATE_IDKDSHABILITADO, $params);
	}

	private function checkForTransfers($CDFILIAL) {
		$params = array(':CDFILIAL' => $CDFILIAL);
		$transfers = $this->connection->fetchAll(self::CHECK_FOR_TRANSFERS, $params);
		if (!empty($transfers)) {
			return $this->KDSTransferController->handleTransfer($transfers);
		} else {
			return array();
		}
	}

	public function compareTimers($a, $b){
		return strcmp($b['NREXIBPROD'], $a['NREXIBPROD']);
	}

	public function generateRequestKey($order){
		return $order['CDFILIAL'] . '-' . $order['NRPEDIDOFOS'] . $order['items'][0]['NRITPEDIDOFOS'];
	}

	public function insertItemsToShowOnPedidoTemp(){

		$sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

		$localQuery = $this->query;
		if(!isset($this->OLDDATE)){
			$this->OLDDATE = array('OLDDATE' => null);
		}

		do {
           
            try {
                $this->connection->executeQuery($localQuery::UPDATE_ITPEDIDOFOS_TO_SHOW_ON_FRONT, $this->OLDDATE);
                $sucess = true;
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);
		
		if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }

		$this->OLDDATE = $this->connection->fetchAssoc("SELECT CONVERT(DATETIME, GETDATE(), 121) AS OLDDATE");

	}

	private function handleOrdersToSend($CDFILIAL) {

		//$this->updatePedidoTemp();
		$this->insertItemsToShowOnPedidoTemp();

		$localQuery = $this->query;
		$this->connection->executeQuery($localQuery::UPDATE_STATUS_UPDATE_PEDIDO_KDS_TEMP);

		$orders = $this->KDSManagerController->getOrders();
		
		// $start = microtime(true);
		// echo 'Processando as vendas trazidas do banco de dados'."\n\n\n";
		// echo '---------------------------------------------------------';		
		$orders = $this->buildOrdersToSend($orders);
		// echo "\n";
		// echo '--------------------------------------------------------------------------'."\n";
		// echo 'Fim do processamento as vendas trazidas do banco de dados: '.(microtime(true)-// $start)."s\n\n";

		// envia os dados pro cliente
		if(!empty($orders['data'])){
			$this->sendToBrowser($orders);
		}

		//tambem usand a trigger
		$this->getStatisticsForAllClients($CDFILIAL);

		//$this->deletePedidoTemp();
		$this->connection->executeQuery($localQuery::DELETE_TREATED_UPDATE_PEDIDO_KDS_TEMP);

	}

	public  function getStatisticsForAllClients($CDFILIAL){
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);
		$getExibEstatics = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_IDEXIBESTATIS, $paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.

		// envia estatisticas
		$params = array(
			':CDFILIAL' => 'T', // T de todos
			':CDSETOR'  => 'T', // T de todos
			':IDTIPOSETOR' => 'E' 
		);

		if (isset($getExibEstatics[0]) && $getExibEstatics[0]["IDEXIBESTATIS"] == "S") {
			// envia estatisticas
			$newStatistics = $this->connection->fetchAll(\Util\Query::GET_STATISTICS, $params);
			if (count($newStatistics) > 0) {
				// indexa por client key
				$indexedNewStatistics = $this->indexStatisticsByClientKey($newStatistics);
				// cria um payload e envia pro socket sendStatistics
				$sendStatistics = array(
					'event' => 'sendStatistics',
					'data'  => $indexedNewStatistics
				);
				// Este $sendStatistics chega no WebSocket como sendo o payload
				$this->sendToBrowser($sendStatistics);
				$this->countStatistics = false;
			}
		} else {
			// Envia as estatísticas zeradas, caso o campo IDEXIBESTATIS seja setado como "N" durante o funcionamento das lojas
			if (!$this->countStatistics) {
				$params = array(
					':CDFILIAL' => $CDFILIAL
				);
				$kdsSectors = $this->connection->fetchAll(\Util\Query::GET_CDSETOR_BY_FILIAL, $params);
				foreach ($kdsSectors as $sector) {
					$params = array(
						':CDFILIAL' => $CDFILIAL, // T de todos
						':CDSETOR'  => $sector['CDSETOR']
					);
					$newStatistics = $this->connection->fetchAll(\Util\Query::GET_CLEAR_STATISTICS, $params);

					$indexedNewStatistics = $this->indexStatisticsByClientKey($newStatistics);
			
					if (!empty($indexedNewStatistics)) {
						$packageToSend = array(
							'event' => 'sendStatistics',
							'data'  => $indexedNewStatistics
						);
			
						// Daqui que saem os dados para o getStatistics do Socket
						$this->sendToBrowser($packageToSend);
					}
					$this->countStatistics = true;
				}
			}
		}
	}

	private function deleteItemsInFront($itemsToDeleteInFront) {
		if (sizeof($itemsToDeleteInFront) > 0) {
			$itemsToDeleteIndexedByOrderKey = $this->indexItemsToDeleteByOrderKey($itemsToDeleteInFront);
			$itemsToDeleteIndexedByClientKey = $this->indexArrayByClientKey($itemsToDeleteIndexedByOrderKey);

			// envia pro websocket os itens que devem ser apagados do frontend
			$packageToWebSocket = array(
				'event' => 'deleteOrders',
				'data'  => $itemsToDeleteIndexedByClientKey
			);
			$this->sendToBrowser($packageToWebSocket);
		}
	}

	private function indexStatisticsByClientKey($statistics){
		$returnArray = array();
		foreach ($statistics as $current) {
			$clientKey = $current['CDFILIAL'] . '-' . $current['CDSETOR'];
			$returnArray[$clientKey] = $current;
		}
		return $returnArray;
	}

	public function indexItemsToDeleteByOrderKey($orders){
		// pedidos indexados por chave
		$currentOrders = array();

		foreach ($orders as $currentOrder) {
			$orderKey = $currentOrder['CDFILIAL'] . '-' . $currentOrder['NRPEDIDOFOS'] . $currentOrder['NRITPEDIDOFOS'];
			$currentOrders[$orderKey] = $currentOrder;
			// coloca chave do item e chave do client (browser) dentro do array de itens
			$currentOrders[$orderKey]['orderKey'] = $orderKey;
			$currentOrders[$orderKey]['clientKey'] = $currentOrder['CDFILIAL'] . '-' . $currentOrder['CDSETOR'];
		}
		return $currentOrders;
	}

	public function sendToBrowser($packageToSend){
		$this->socketClient->send(json_encode($packageToSend));
	}

	private function buildOrdersToSend($orders) {
		// pedidos que foram alterados, serão enviados para o frontend
		$ordersThatChanged = array();
		// indexa o array pela chave do pedido
		$currentOrders = $this->indexOrdersByOrderKey($orders);

		foreach ($currentOrders as $key => $order) {
			if ($this->orderStash->orderHasChanged($order)) {
				// preenche array com os pedidos e itens que mudaram para mandar pro frontend
				$ordersThatChanged[$key] = $order;
				$this->orderStash->insertOrder($order);
			}
		}

		// verifica se os itens estão produzidos para a expedição
		$ordersThatChanged = $this->handleProducedItems($ordersThatChanged);

		//indexa pedidos pela chave do cliente
		$ordersIndexedByClientKey = $this->indexArrayByClientKey($ordersThatChanged);

		// verifica os itens para definir se o pedido é prioridade
		

		return array(
			'event' => 'sendChanges',
			'data'  => $ordersIndexedByClientKey
		);
	}

	private function handleProducedItems($orders) {
		$returnArray = array();
		foreach ($orders as $order) {
			if ($order['IDTIPOSETOR'] == "E") {
				// para cada item da expedição, verifica se sua composição está pronta
				$itemsArray = array();
				foreach ($order['items'] as $item) {
					$item = $this->KDSManagerController->updateExpeditionStatus($order['CDFILIAL'], $item);
					array_push($itemsArray, $item);
				}
				$order['items'] = $itemsArray;
			}
			$returnArray[$order['orderKey']] = $order;
		}
		return $returnArray;
	}

	public function indexOrdersByOrderKey($orders){
		// pedidos indexados por chave
		$currentOrders = array();

		foreach ($orders as $request) {
			$orderKey = $this->generateRequestKey($request);
			$currentOrders[$orderKey] = $request;
			// coloca chave do item e chave do client (browser) dentro do array de itens
			$currentOrders[$orderKey]['orderKey'] = $orderKey;
			$currentOrders[$orderKey]['clientKey'] = $request['CDFILIAL'] . '-' . $request['items'][0]['CDSETOR'];
		}
		return $currentOrders;
	}

	public function indexArrayByClientKey($orders){
		$ordersIndexedByClientKey = array();
		foreach ($orders as $item) {
			if (empty($ordersIndexedByClientKey[$item['clientKey']])){
				// se ainda não existe o array de itens para este cliente, cria ele
				$orderArray = array(
					$item['orderKey'] => $item
				);
				$ordersIndexedByClientKey[$item['clientKey']] = $orderArray;
			} else {
				// se já existe o array, somente adiciona o próximo
				$ordersIndexedByClientKey[$item['clientKey']][$item['orderKey']] = $item;
			}
		}
		return $ordersIndexedByClientKey;
	}

	const CHECK_FOR_TRANSFERS = "
		SELECT CDFILIALOLD, NRVENDARESTOLD, NRCOMANDAOLD, NRPRODCOMVENOLD, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, NRSEQPRODCUP, NRMESA, NRLUGARMESAIT
		  FROM PEDIDOALT
		 WHERE CDFILIALOLD = :CDFILIAL
	";
	public function getFrozen(){
		$localQuery = $this->query;
		return $this->connection->fetchAssoc($localQuery::GET_IDKDSHABILITADO)['IDKDSHABILITADO'] == 'N';
	}

	public function uptateTimeToShow ($paidOrder) {
		/* Caso a loja utilize o parametro IDCONDPEDPAG para exibir 
		somente pedidos pagos no KDS, Refaz o DTHREXIBICAOPROD para quando o pedido for pago */
		foreach ($paidOrder as $order) {
			$params['CDLOJA'] = $order['CDLOJA'];
			$params['NRPEDIDOFOS'] = $order['NRPEDIDOFOS'];
			$conditionOrderToPayment = $this->connection->fetchAssoc($this->query::GET_IDCONDPEDPAG_BY_LOJA, $params);
			if (!empty($conditionOrderToPayment) && ($conditionOrderToPayment['IDCONDPEDPAG'] == 'S') && (!empty($order['NRSEQVENDA'] || !empty($order['NRSEQMOVDLV'])))) {
				try {
					$itensITPEDIDOFOS = $this->connection->fetchAll($this->query::GET_ITEMS_ITPEDIDOFOS, $params);
					$itensToUpdate = $this->KDSManagerController->redefineOrderTimers($itensITPEDIDOFOS);
					foreach ($itensToUpdate as $ite) {
						$params = array(
							'NRPEDIDOFOS' => $ite['NRPEDIDOFOS'],
							'NRITPEDIDOFOS' => $ite['NRITPEDIDOFOS'],
							'DTHREXIBICAOPROD' => $ite['DTHREXIBICAOPROD']
						);
						$this->connection->executeQuery($this->query::UPDATE_DTHREXIBICAOPROD_BY_NRPEDIDOFOS, $params);
					}
					// $message = "Pedido " . $order['CDSENHAPED'] . ' pagamento identificado';
					// $this->util->logExecution('condiciona_pedido_pagamento', $message);
				} catch (\Exception $e) {
					$this->util->logException($e->getMessage());
				}
			}
		}
	}

	public function checkAllEntries() {
		// cria ou atualiza todas as triggers, tabelas e campos necessarios para o cuncionamento do KDS

		$check_tables = ["PEDIDO_KDS_TEMP", "PEDIDO_KDS_TEMP_ESPERA", "KDS_TEMP_SETOREST", 'UPDATE_PEDIDO_KDS_TEMP'];

		$check_filelds = [
			['QTDRODIZIO', 'CREATE_FIELDS_RODIZIO'],
			["NRTEMPOATRASOEXP","ALTERS_VERSION_350"],
			['NRSEQOPERACAOPK', 'CRIA_CAMPO_PK_KDSOPERACAOTEMP'],
			// Campos criados internamente pelo KDS apenas até o FOSTAB passar a cria-los automaticamente.
			// Remover os dois campos abaixo daqui assim que possivel (com as 4 queries respectivas).
			['IDIMPRPRODPICK', 'CREATE_FIELD_IDIMPRPRODPICK'],
			['IDEXIBESTATIS', 'CREATE_FIELD_IDEXIBESTATIS']
		];
		$check_triggers = [
			"TGR_PEDIDOFOS", "TGR_ITPEDIDOFOS", "TGR_ITPEDIDOFOSREL", "TRIGGER_KDSMENSAGEM_ON_DELETE", "TGR_AI_ITCOMANDAVEN_INS_PED_KDS_TMP",
			"TGR_AI_ITCOMANDAEST_INS_UP_PED_KDS_TMP", "TGR_AI_VENDA_INS_PED_KDS_TMP", "TGR_FIU_PEDIDOFOS_INS_UPD_PED_KDS_TMP", 
			"TGR_FIU_ITPEDIDOFOS_INS_UPD_PED_KDS_TMP", "TGR_FIU_ITPEDIDOFOSREL_INS_UPD_PED_KDS_TMP_2", "TGR_FIU_ITPEDIDOFOSEST_INS_UPD_PED_KDS_TMP",
			"TGR_MOVCAIXADLV", "TGR_MOVCAIXA"
		];

		$check_entities_Norm = [
			['GET_TGR_MOVCAIXA082021', 'DROP_TGR_MOVCAIXA082021'],
			['GET_TGR_MOVCAIXADLV062021', 'DROP_TGR_MOVCAIXADLV062021'],
			['GET_TGR_MOVCAIXADLV082021', 'DROP_TGR_MOVCAIXADLV082021'],
			['GET_TGR_MOVCAIXADLV', 'DROP_TGR_MOVCAIXADLV'],
			['GET_TGR_MOVCAIXA', 'DROP_TGR_MOVCAIXA'],
			[$this->pos0000196913, 'DROP_PK_KDSOPERACAOTEMP'],
			[$this->pos0000196913, 'ALTERA_NRSEQOPERACAO_KDSOPERACAOTEMP'],
			[$this->pos0000196913, 'ALTERA_PK_KDSOPERACAOTEMP'],
			["GET_TRIGGER_FIU_ITPEDIDOFOSREL_INS_UPD_PED_KDS_TMP","DROP_TGR_FIU_ITPEDIDOFOSREL_INS_UPD_PED_KDS_TMP"]
		];

		try {
			$this->util->logExecution('Atualizando triggers');
			foreach ($check_tables as $actual_table) {
				$count_obj = $this->connection->fetchAll($this->query::GET_TABLE, array('TABLE' => $actual_table));
				if (!$count_obj || count($count_obj) == 0) {
					$this->connection->executeQuery($this->query::getConst($actual_table));
				}
			}
	
			foreach ($check_filelds as $actual_fileld) {
				$count_obj = $this->connection->fetchAll($this->query::GET_FIELD, array('FIELD' => $actual_fileld[0]));
				if (!$count_obj || count($count_obj) == 0) {
					$this->connection->executeQuery($this->query::getConst($actual_fileld[1]));
				}
			}
			// apaga as triggers Antigas
			foreach ($check_entities_Norm as $key => $check_entities_NormValue) {
				$check = $check_entities_NormValue[0];
				$command = $check_entities_NormValue[1];
				$shouldLog = isset($check_entities_NormValue[2]) ? $check_entities_NormValue[2] : true;
				$count_obj = [];
				if($check && (!($check === true || $check === false))) {
					$count_obj = $this->connection->fetchAll($this->query::getConst($check));
				}
	
				if (count($count_obj) != 0 || ($check === null || $check === true)) {
					$this->connection->executeQuery($this->query::getConst($command));
					$this->KDSLogs->log("Trigger [$command] dropada do banco.", [], dirname(__FILE__).'\\'.basename(__FILE__));
				}
			}
	
			foreach ($check_triggers as $actual_trigger) {
				$count_obj = $this->connection->fetchAll($this->query::GET_TRIGGER, array('TR' => $actual_trigger));
				$this->connection->executeQuery($this->query::getConst($actual_trigger));
			}

			$this->util->logExecution('Triggers atualizadas com sucesso');

		} catch (\Exception $e) {
			$this->util->logException($e->getMessage());
		}
	}

	private function clearStatistics () {

		try {

			if (!$this->hora_limpeza_estatistica) {
				$this->hora_limpeza_estatistica = $this->instanceManager->getParameter('HORA_LIMPEZA_ESTATISTICA');
				$this->hora_limpeza_estatistica = empty($this->hora_limpeza_estatistica) || intval($this->hora_limpeza_estatistica) < 0 || 
				intval($this->hora_limpeza_estatistica) > 23 ? 
				1 : intVal($this->hora_limpeza_estatistica);
			}
	
			if($this->currentDay != date('d')){
				if(intVal(date('H')) >= $this->hora_limpeza_estatistica && intVal(date('H')) < ($this->hora_limpeza_estatistica + 1) ){
					$this->connection->executeQuery('DELETE FROM KDS_TEMP_SETOREST; DELETE FROM ITPEDIDOFOSEST;');
					$this->currentDay = date('d');
					$this->environment->clearSession();
					$this->util->logExecution('Estatisticas apagadas com sucesso');
				}
			}

		} catch (\Exception $e) {
			$this->util->logException($e);
		}
	}

	private function clearOldOrders () {
		try {
			$this->intervalo_limpeza = $this->intervalo_limpeza ? $this->intervalo_limpeza : $this->instanceManager->getParameter('INTERVALO_LIMPEZA_PEDIDOS');
			if (($this->intervalo_limpeza != false) && ($this->currentHour !== date('H'))){
				$this->intervalo_limpeza = empty($this->intervalo_limpeza) || (intval($this->intervalo_limpeza) < 1) ? 
												24 : intVal($this->intervalo_limpeza);

				$params['INTERVALO'] = ($this->intervalo_limpeza * -1);
				$this->currentHour = date('H');
				$totalOrdersToRemove = $this->connection->fetchAssoc($this->query::COUNT_CLEAN_ORDERS, $params);
				if ($totalOrdersToRemove['TOTAL'] > 0) {
					$this->connection->executeQuery($this->query::CLEAN_ORDERS_STATISTICS, $params);
					$this->connection->executeQuery($this->query::CLEAN_ORDERS, $params);
					$message = $totalOrdersToRemove['TOTAL'] . " pedidos apagados após o intervalo de ". ($this->intervalo_limpeza). " sem serem expedidos";
					$this->util->logExecution($message);
				} else {
					$this->util->logExecution('Não há pedidos a serem removidos');
				}
			}
		} catch (\Exception $e) {
			$this->util->logException($e);
		}
	}
}