function ListController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService,
	$filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator, DecodeValueFieldService) {

	// BEGIN: Initialize
	var self = this;
	self.i18n = ScreenService.i18n;

	var widgetAccessorDescriptor = {
		get: function () {
			return $scope.widget || $scope._currentPopup;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);

	function extendBehavior() {
		Util.decorateFromConstructor(self, OrderByable);
		Util.decorateFromConstructor(self, Searchable);
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

	var moment = $window.moment;
	var allowMultiSelect = null;
	$scope.filtered = [];

	this.getAllowMultiSelect = function getAllowMultiSelect() {
		return allowMultiSelect;
	};

	eventAggregator.publish("OnListStartUp", {
		widget: $scope.widget
	});
	// END: Initialize

	// Floating Control defaults
	var floatingControlDefaults = {
		pdfAction: false,
		xlsAction: false,
		customizationAction: {
			filter: true,
			layout: false,
			view: false
		}
	};
	self.updateFloatingControlDefaults = Util.buildDebounceMethod(function(options) {
		self.widget.floatingControl = Util.extend(options, self.widget.floatingControl || {});
	}, 100);
	self.updateFloatingControlDefaults(floatingControlDefaults);

	// BEGIN: Search
	ZhFeatures.registerFeatureUsage({
		'controller': 'ListController',
		'widgetName': $scope.widget ? $scope.widget.name : ''
	});

	this.search = function(gridRow) {
		self.widget.searchCriteria = self.widget.searchCriteria || {};
		self.widget.searchList = self.widget.searchCriteria._ALL;
		var rowVisibility = true;
		if (Util.objectHasAnyValue(self.widget.searchCriteria)) {
			rowVisibility = self.checkRowVisibilityFromCriteria(gridRow, self.widget.searchCriteria);
		}
		if (rowVisibility && !$scope.widget.dataSource.getStorageStrategy()) {
			if (!$scope.widget.applyFilters) {
				Util.extend($scope.widget, new Filterable(ZhUtil, Query));
			}

			rowVisibility = $scope.widget.applyFilters(gridRow, $scope.widget.getDataSourceFilter());
		}
		gridRow.__notSelectable = !rowVisibility;
		return rowVisibility;
	};

	this._searchWithPaginationReload = Util.buildDebounceMethod(_searchWithPaginationReload, 1000);

	function _searchWithPaginationReload(widget, callback) {
		var promise;
		if (Util.isArray(self.widget.fields) && self.widget.fields.length) {
			promise = widget.reload(ZhUtil.getLikeAllFilter(self.widget, self.widget.searchCriteria || {}));
		} else {
			promise = ZHPromise.reject();
		}
		if (callback) {
			callback(promise);
		}
	}

	var _resizeGrid = Util.buildDebounceMethod(function() {
		self.widget.resizeGridColumns();
	}, 200);

	if (!self.isGantt(self.widget)) {
		$scope.$watch(function() {
			return self.widget.searchCriteria;
		}, function(current, old) {
			if (current !== old) {
				if (self.widget.resetInfinityScroll) {
					self.widget.resetInfinityScroll();
				}
				searchFromSearchCriteria();
			}
		}, true);
	}

	var searchFromSearchCriteria = Util.buildDebounceMethod(function() {
		if (!self.widget._noReloadFromSearchCriteria) {
			self.widget.searchList = self.widget.searchCriteria && self.widget.searchCriteria._ALL || undefined;
			updateGroupedData();
			if (self.widget.searchWithPagination) {
				self._searchWithPaginationReload(self.widget, function(searchPromise) {
					searchPromise.then(updateGroupedData);
				});
			}
			if (self.widget.resizeGridColumns) { // new grid
				_resizeGrid();
			}
		} 
		self.widget._noReloadFromSearchCriteria = false;
	}, 200);
	// END: Search

	// BEGIN: Toggle
	this.deselectAll = function deselectAll(widget) {
		var event = widget.unSelectAll();
		if (!event || !event.isDefaultPrevented()) {
			widget.dataSource.clearCheckedRows();
			if (!$scope._currentPopup) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}
		}
	};

	$rootScope.$on("ui.deselectAll", function(event, widget) {
		self.deselectAll(widget);
	});

	this.onLongTap = function(row) {
		if (row.__group) return;

		if (this.widget.allowSelectOnLongTap) {
			this.toggle(row);
		}
	};


	this.toggle = function(row) {
		var widget = self.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}

		if (!event || !event.isDefaultPrevented()) {

			var index = getIndex(row, $scope);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}

			if (self.isTree(self.widget)) {
				self.handleDependentRows(row, widget);
			}

			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}

		}
	};

	$scope.widget.toggle = self.toggle;

	this.getVisibleCheckedRows = function() {
		return (self.widget.groupedData || self.widget.dataSource.data).filter(function(row) {
			return row.__isSelected;
		});
	};

	this.toggleAll = function() {
		if (self.widget.loadMore && self.getVisibleCheckedRows().length === 0) {
			ScreenService.confirmMessage('There are more rows that are not being displayed. Do you want to load them before selecting/unselecting all? (This may take some time)', 'CONFIRMATION', self._loadAllDataAndToggle, self._toggleAll);
		} else {
			self._toggleAll();
		}
	};

	this._toggleAll = function(useDatasourceData) {
		var checkedRows = self.getVisibleCheckedRows();
		var rows = (!useDatasourceData && self.widget.groupedData) || self.widget.dataSource.data;
		if (checkedRows.length > 0) {
			self.widget.unSelectAll(rows);
		} else {
			self.widget.selectAll(rows);
		}
		self.widget.updateCheckedRows();
	};

	this._loadAllDataAndToggle = function() {
		var dataSourceFilters = self.widget.getDataSourceFilter();
		dataSourceFilters = dataSourceFilters.concat(ZhUtil.getLikeAllFilter(self.widget, self.widget.searchCriteria));
		self.widget.dataSource.load(dataSourceFilters, 1, Number.MAX_SAFE_INTEGER).then(function() {
			self.widget.dataSource.setCurrentPage(Math.ceil(self.widget.dataSource.data.length / (self.widget.itemsPerPage || self.widget.dataSource.itemsPerPage))); // set current page to highest number
			self.widget.infinityScroll();
			self._toggleAll(true);
		});
	};

	this.widget.toggle = this.toggle;
	// END: Toggle

	// BEGIN: Others functions
	$scope.openRow = function(row, widget) {
		if (row.__group) return;

		if ((widget.showCheckbox && widget.dataSource.getCheckedRows().length) || allowMultiSelect === widget || widget.allowMultiSelect) {
			self.toggle(row);
		} else {
			widget.setCurrentRow(row);
		}
		widget.dataSource.updateCheckedRows();
	};

	$scope.openRowInnerList = function(row, widget) {
		if (!Util.isUndefined(widget.currentRow) &&
			widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
			delete widget.currentRow;
		} else {
			$scope.openRow(row, widget);
		}
	};

	function getIndex(row, scope) {
		return self.widget.dataSource.checkedRows.indexOf(row);
	}

	$scope.scrollTo = function(evt) {
		$timeout(function() {
			var item = $(evt.target).closest('li');
			var itemTop = item.position().top;
			var itemBottom = itemTop + item[0].clientHeight;
			var scroller = item.closest("[zh-scroller]");
			var top = 0;
			if (itemTop < 0 || itemBottom > scroller.height()) {
				if (itemTop < 0) {
					top = scroller.scrollTop() + itemTop - 10;
				} else {
					top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
				}
				scroller.animate({
					scrollTop: top
				}, 500);
			}
		}, 100, true);
	};

	$rootScope.$on("ui.allowMultiSelect", function(event, widget) {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = widget;
	});

	$rootScope.$on("ui.disableMultiSelect", function() {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = null;
	});

	this.getGroupHeaderText = function getGroupHeaderText(row) {
		var groupHeaderText = "";
		if (row) {
			var middleText = row.__groupLabel && row.__groupValue ? ": " : "";
			var groupLabel = row.__groupLabel || "";
			var groupValue = row.__groupValue || "";
			groupHeaderText = groupLabel + middleText + groupValue;
		}

		return groupHeaderText;
	};

	this.getGroupFooterTitle = function getGroupFooterTitle(row) {
		var groupFooterTitle = "";
		if (row) {
			if (row.__groupValue) {
				var middleText = row.__groupLabel && row.__groupValue ? ": " : "";
				var groupLabel = row.__groupLabel || "";
				var groupValue = row.__groupValue || "";
				groupFooterTitle = ScreenService.i18n("Total ") + "(" + groupLabel + middleText + groupValue + ")";
			} else {
				groupFooterTitle = ScreenService.i18n("TOTAL");
			}
		}

		return groupFooterTitle;
	};

	// END: Others functions

	// BEGIN: Conditional actions control

	var hasConditionalActions = false;
	var leftActions = [];
	var rightActionsGrid = [];
	var rightActionsList = [];
	var rightActions = [];
	var defaultActionsValue = function() {
		$scope.visibleLeftActions = leftActions.length;
		$scope.visibleRightActions = rightActions.length;
		$scope.leftActions = leftActions;
		$scope.rightActions = rightActions;
	};
	var buildGridActions = function() {
		hasConditionalActions = false;
		leftActions = $scope.widget.getActionsByVisibility('grid-left');
		rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
		rightActionsList = $scope.widget.getActionsByVisibility('list-right');
		rightActions = rightActionsGrid.concat(rightActionsList);

		$scope.widget.actions.forEach(function(item) {
			if (item.condition) {
				hasConditionalActions = true;
			}
		});

		if (!hasConditionalActions) {
			defaultActionsValue();
		}
	};
	$scope.buildGridActions = buildGridActions;
	buildGridActions();

	this.widget.updateGridActions = function updateGridActions() {
		_updateGridActions();
		templateManager.updateTemplate();
	};

	function updateGridActionsOnRow(row) {
		row = row || {};
		ApplicationContext.registerInstance('row', row);

		row.__rightActions = filterActionsAccordingToCondition(rightActions);
		row.__leftActions = filterActionsAccordingToCondition(leftActions);
		$scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
		$scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);

		ApplicationContext.removeInstance('row');
	}

	function _updateGridActions() {
		if (hasConditionalActions) {

			$scope.visibleLeftActions = 0;
			$scope.visibleRightActions = 0;

			if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
				$scope.widget.dataSource.data.forEach(updateGridActionsOnRow);
			}
		}
	}

	function filterActionsAccordingToCondition(actions) {
		return actions.filter(function(action) {
			return !action.condition || ZhUtil.evaluate("@" + action.condition + "@");
		});
	}

	// END: Conditional actions control

	// BEGIN: Order by

	this.changeOrder = function(column) {
		if (!!column.disableOrdering) return;

		if (self.widget.order == column.name && self.widget.reverse) {
			self.widget.order = undefined;
			self.widget.reverse = false;
		} else {
			self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
			self.widget.order = column.name;
		}

		publishAfterChangeOrderEvent();
		self.updateOrderBy();
	};

	this.updateOrderBy = function() {
		self.updateOrderByColumns(self.widget);
		var dataSourceFilters = (self.widget.dataSourceFilter || []).filter(function(filter) {
			return filter.operator != Query.ORDER;
		});

		var orderByFilter = ZhUtil.getOrderByFilter(self.getOrderByColumns());
		if (orderByFilter.length > 0 && orderByFilter[0].name.length > 0) {
			dataSourceFilters = dataSourceFilters.concat(orderByFilter);
		}

		if (!angular.equals(dataSourceFilters, self.widget.dataSourceFilter)) {
			self.widget.dataSourceFilter = dataSourceFilters;

			if (self.widget.resetInfinityScroll) {
				self.widget.resetInfinityScroll();
			}
			self.widget.reload(); // keep current page
		}
	};

	function publishAfterChangeOrderEvent() {
		if (self.widget.afterChangeOrder) {
			self.widget.afterChangeOrder();
		}
	}

	// END: Order by

	// BEGIN: Group and summarization
	$scope.fieldOrder = function(field) {
		return (field.fixed ? '0' : '1') + ('000' + field.sequenceInList).substr(-4);
	};

	$scope.widget.groupedData = [];

	// initialize widget.groups
	updateWidgetGroups();

	if ($scope.widget) {
		$scope.widget.updateGroups = function updateGroups(){
			updateWidgetGroups();
			updateGroupedData();
		};
		$scope.widget.clearSearchList = $scope.widget.clearSearchCriteria = clearSearchCriteria;
		$scope.widget.setSearchList = $scope.widget.setSearchCriteria = setSearchCriteria;
	}

	function updateWidgetGroups() {
		$scope.widget.groups = $scope.widget.fields.filter(function(item) {
			if (item.grouped) item.isVisible = false;
			return item.aclIsVisible && item.grouped;
		}).sort(function(a, b) {
			return a.sequenceInList - b.sequenceInList;
		});

		if ($scope.widget.groups.length) {
			self.updateOrderBy();
		}
	}

	function clearSearchCriteria() {
		self.widget.searchCriteria = {};
		self.widget.searchList = "";
	}

	function setSearchCriteria(searchCriteria, forceReload) {
		if (!forceReload) {
			self.widget._noReloadFromSearchCriteria = true;
		}

		clearSearchCriteria();
		if (searchCriteria) {
			searchCriteria.forEach(function(field) {
				var fieldValue = field.value;
				if (fieldValue[fieldValue.length - 1] == '%') {
					fieldValue = fieldValue.substring(0, (fieldValue.length - 1));
				}
				if (fieldValue[0] == "%") {
					fieldValue = fieldValue.substring(1, fieldValue.length);
				}
				self.widget.searchCriteria[field.name] = fieldValue;
			});
		}

		searchFromSearchCriteria(self.widget.searchCriteria);
	}

	var oldItemsPerPage = $scope.widget.itemsPerPage;
	var oldHasPagination = $scope.widget.hasPagination;

	$scope.$watchCollection('widget.groups', function(newValue, oldValue) {
		if (newValue != oldValue) {
			if (newValue && newValue.length >= 1) {
				$scope.widget.itemsPerPage = 0;
				$scope.widget.hasPagination = false;
			} else {
				$scope.widget.itemsPerPage = oldItemsPerPage;
				$scope.widget.hasPagination = oldHasPagination;
			}
			$scope.widget._gridResize = true; // forces the grid element to recalculate column widths

			self.updateOrderBy();
			updateGroupedData();
		}
	});

	$scope.$watch(function() {
		return ZhUtil.clearRow($scope.widget.currentRow);
	}, function() {
		updateGridActionsOnRow($scope.widget.currentRow);
	}, true);

	$scope.$watchCollection('widget.dataSource.data', function() {
		_updateGridActions();
		updateGroupedData();
	});

	var groups = [];
	var summaryFields = [];
	var summary = {};

	function resetFooterVariables(group) {
		group.footer.count = 0;
		summaryFields.forEach(function(field) {
			group.footer[field.name] = {};
			group.footer[field.name].sum = 0;
			group.footer[field.name].count = 0;
		});
	}

	function addGroupHeader(group, index, groupOpened) {
		var groupHeaderRow = {
			__group: true,
			__groupHeader: true,
			__groupIndex: index,
			__groupLabel: ScreenService.i18n(group.label),
			__groupValue: !group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil),
			__groupOpened: Util.isUndefinedOrNull(groupOpened) ? true : groupOpened,
			__groupHeaders: []
		};
		// add header for outer groups to the added row
		for (var i = 0; i < index; i++) {
			groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
		}
		group.lastGroupHeaderRow = groupHeaderRow;
		$scope.widget.groupedData.push(groupHeaderRow);
	}

	function summaryData(group, row, isGroup) {
		var field, value, lastValue, _group;
		for (var i = 0; i < summaryFields.length; i++) {
			field = summaryFields[i];
			if (isGroup) {
				lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
				_group = group.footer;
			} else {
				lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
				_group = group;
			}

			if (field.expression == 'COUNT') {
				row[field.name] = _group[field.name].count;
			} else if (field.expression == 'SUM') {
				value = _group[field.name].sum;
				row[field.name] = (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
			} else if (field.expression == 'AVG') {
				value = _group[field.name].sum / _group[field.name].count;
				row[field.name] = (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
			}
		}
	}

	function addGroupFooters(groupIndex) {
		if (summaryFields.length === 0) return;

		for (var i = groups.length - 1; i >= groupIndex; i--) {
			if (groups[i].footer.count > 0) {
				var groupFooterRow = {
					__group: true,
					__groupFooter: true,
					__groupIndex: i,
					__groupHeaders: [],
					__groupLabel: groups[i].lastGroupHeaderRow.__groupLabel,
					__groupValue: groups[i].lastGroupHeaderRow.__groupValue
				};

				// add header for outer groups
				for (var g = 0; g < i; g++) {
					groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
				}

				summaryData(groups[i], groupFooterRow, true);

				$scope.widget.groupedData.push(groupFooterRow);
			}
		}
	}

	function addSummary() {
		if (summaryFields.length === 0 || groups.length === 0) return;

		var summaryRow = {
			__group: true,
			__groupFooter: true,
			__groupSummary: true
		};

		summaryData(summary, summaryRow);

		$scope.widget.groupedData.push(summaryRow);
	}

	function calcSummary(row) {
		groups.forEach(function(group) {
			group.footer.count += 1;
			if (summaryFields.length > 0) {
				summaryFields.forEach(function(field) {
					if (!Util.isUndefined(row[field.name])) {
						group.footer[field.name].count += 1;
					}
					if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
						group.footer[field.name].sum += parseFloat(row[field.name]);
					}
				});
			}
		});

		summaryFields.forEach(function(field) {
			if (Util.isUndefined(summary[field.name])) {
				summary[field.name] = {
					count: 0,
					sum: 0
				};
			}

			if (!Util.isUndefined(row[field.name])) {
				summary[field.name].count += 1;
			}
			if ((field.expression == 'SUM' || field.expression == 'AVG') && !isNaN(row[field.name])) {
				summary[field.name].sum += parseFloat(row[field.name]);
			}
		});
	}

	function initSummaryFields() {
		summaryFields = $scope.widget.fields.filter(function(field) {
			return !!field.expression && !field.grouped;
		});

		// reset summary variable
		summary = {};
	}

	function initGroups() {
		groups = [];
		$scope.widget.groups.forEach(function(group, index) {
			var newGroup = {
				field: group,
				name: group.name,
				label: group.label,
				lastValue: undefined,
				footer: {}
			};
			resetFooterVariables(newGroup);

			groups.push(newGroup);
		});
	}

	function getOrderedDataByColumns(dataSet, orderByColumns) {
		return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
	}

	function updateGroupedData() {
		if (Util.isUndefined($scope.widget.dataSource) || Util.isEmptyOrBlank($scope.widget.dataSource.data)) {
			$scope.widget.groupedData = [];
		} else {
			var filteredData = $scope.widget.dataSource.data;
			if (!$scope.widget.dataSource.getStorageStrategy() || !$scope.widget.searchWithPagination) {
				filteredData = filteredData.filter(self.search);
			}

			if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
				$scope.widget.groupedData = getOrderedDataByColumns(filteredData, self.getOrderByColumns());

				// remove group headers
				$scope.widget.groupedData.forEach(function(row) {
					delete row.__groupHeaders;
				});
			} else {
				$scope.widget.groupedData = [];

				initSummaryFields();
				initGroups();

				var orderedData = getOrderedDataByColumns(filteredData, self.getOrderByColumns());
				orderedData.forEach(function(row, index) {
					var groupBreak = false;
					groups.forEach(function(group, index) {
						var headerValue = row[group.name];
						var modelHeaderValue = group.field && DecodeValueFieldService.model(group.field, headerValue);
						var lastValue =  modelHeaderValue || headerValue;
						if (group.lastValue != lastValue || groupBreak) {
							if (!groupBreak) addGroupFooters(index);
							group.lastValue = lastValue;
							resetFooterVariables(group);

							addGroupHeader(group, index, !$scope.widget.bringGroupsCollapsed);
							groupBreak = true;
						}
					});

					row.__groupHeaders = groups.map(function(group) {
						return group.lastGroupHeaderRow;
					}).filter(function(group){
						return !Util.isUndefined(group);
					});
					$scope.widget.groupedData.push(row);

					calcSummary(row);
				});

				if ($scope.widget.groupedData.length > 0) {
					addGroupFooters(0);
					addSummary();
				}
			}
		}

		if (self.widget._updateVisibleRows) {
			self.widget._updateVisibleRows();
		}
	}

	this.toggleGroup = function(row) {
		row.__groupOpened = !row.__groupOpened;

		if (self.widget._updateVisibleRows) {
			self.widget._updateVisibleRows();
		}
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.getVisibleRowCount = function getVisibleRowCount() {
		var visibleRowsCount = [];
		var widget = $scope.widget;

		if(widget._getVisibilityParams) {
			widget._getVisibilityParams();
			if (widget.groupedData) {
				var dataLength = widget.groupedData.length;
				var rowCount = Math.min(dataLength, widget._visibilityParams.maxVisibleRows);

				visibleRowsCount = Util.range(rowCount);

				self._rows = self.getDataSliceFromRowCount(widget.groupedData, rowCount);

				if (widget._hiddenRows !== dataLength - rowCount) {
					widget._hiddenRows = dataLength - rowCount;
					widget._calcScrollAreas();
				}
			}
		}

		return visibleRowsCount;
	};

	$scope.widget._updateVisibleRows = Util.buildDebounceMethod(function() {
		if ($scope.widget.groupedData && $scope.widget.groupedData.length > 0) {
			self._rows = self.getDataSliceFromRowCount($scope.widget.groupedData, self._rows? self._rows.length : 0);

			if (Util.isMobile() && $scope.widget._calcScrollAreas) {
				$scope.widget._calcScrollAreas();
			}

			templateManager.updateTemplate();
		}
	}, 200);

	this.getDataSliceFromRowCount = function getDataSliceFromRowCount(data, rowCount) {
		var dataSlice = [];
		if (!data) {
			data = [];
		}
		var topRow = $scope.widget._topRow || 0;

		dataSlice = data.slice(topRow, topRow + rowCount);

		return dataSlice;
	};

	/* STRIPE FOOTER */
	this.stripeFooter = function stripeFooter() {
		var stripeWidget;

		if ($scope.widget && $scope.widget.widgets) {
			stripeWidget = $scope.widget.widgets.filter(function(widget) {
				return ~widget.template.indexOf("stripe.html");
			}).shift();
		}

		return stripeWidget;
	};
}